#---------------------------------
#IMPORTS
#---------------------------------
from astropy.io import fits
import numpy as np 
import glob as g
from skimage.registration import phase_cross_correlation
import matplotlib.pyplot as plt
from photutils import CircularAperture, CircularAnnulus, aperture_photometry
from math import pi, pow
import csv
from astropy.time import Time
from astropy.nddata import CCDData

#---------------------------------
#GET FILES	
#---------------------------------
directToWatch = str(input("Enter the Directory Path where FITS Image Files are located: "))
directoryP = directToWatch
# Add / to end of directory if user does not input it
if directToWatch[-1] != "/":
    directToWatch += "/"
# Find fits files
fits_extensions = ["*.fts", "*.FITS", "*.FIT", "*.fits", "*.fit"]
looper = True
# Loop until we find something
while looper:
    for exti in fits_extensions:
        inputfiles = g.glob(directToWatch+exti)
        # If we find files, then stop the for loop and while loop
        if len(inputfiles) > 0:
            looper = False
            break
    # If we don't find any files, then we need the user to check their directory and loop over again....
    if len(inputfiles) == 0:
        print("Error: .fts files not found in " + directToWatch + ". Please try again.")
        directToWatch = str(input("Enter the Directory Path where FITS Image Files are located: "))
        # Add / to end of directory if user does not input it
        if directToWatch[-1] != "/":
            directToWatch += "/"
            
sortedfilesArr = np.array(sorted(inputfiles))
sortedfiles = list(sortedfilesArr)

#---------------------------------
#FUNCTIONS	
#---------------------------------    
def getPixelValue(x_ref, y_ref, data, refData, apR, innR, outR):
	shift, error, diffphase = phase_cross_correlation(refData, data)
	xOffset = shift[1]
	yOffset = shift[0]
	xCoord, yCoord = x_ref - xOffset, y_ref - yOffset
	position = (xCoord, yCoord)
	aperture = CircularAperture(position, r=apR)
	phot_table = aperture_photometry(data, aperture)
	arr = np.array(phot_table)
	raw_pixel = arr[0][3]
	annulus = CircularAnnulus(position, innR, outR)
	phot_table2 = aperture_photometry(data, annulus)
	arr2 = np.array(phot_table2)
	annulus_sum = arr2[0][3]         
	area1 = pi*pow(outR, 2)
	area2 = pi*pow(innR, 2)
	annulus_area = area1 - area2
	avg_bkgrnd = annulus_sum/annulus_area
	aperture_area = pi*pow(apR, 2)
	total_bkgrnd = avg_bkgrnd*aperture_area
	pixelValue = raw_pixel - total_bkgrnd
	return pixelValue

def getCalibAvg(calibType):
	calibPath = str(input('Enter the directory path to your ' + calibType + ' (must be in their own separate folder): '))  # +"/*.FITS"
	# Add / to end of directory if user does not input it
	if calibPath[-1] != "/":
		calibPath += "/"
	# Check for .FITS files
	inputcalib = g.glob(calibPath + "*.FITS")
	# If none exist, try other extensions
	if len(inputcalib) == 0:
		inputcalib = g.glob(calibPath + "*.FIT")
	if len(inputcalib) == 0:
		inputcalib = g.glob(calibPath + "*.fits")
	if len(inputcalib) == 0:
		inputcalib = g.glob(calibPath + "*.fit")
	if len(inputcalib) == 0:
		inputcalib = g.glob(calibPath + "*.fts")
	if len(inputcalib) == 0:
		print("Error: no " + calibType + " found in " + calibPath + ". Proceeding with reduction WITHOUT " + calibType[0:4] + "fields.")
		calibBool = False
		calibAvg = []
	else:
		calibBool = True
		calibImgList = []
		for calibFile in inputcalib:
			calibData = fits.getdata(calibFile, ext=0)
			calibImgList.append(calibData)
		calibImgs = np.array(calibImgList)
		calibAvg = np.average(calibImgs,axis=0)
	return calibBool, calibAvg, calibPath

def calibrate(files, biasAvg, flatAvg):
	biasAvgCCD = CCDData(biasAvg, unit='adu')
	flatAvgCCD = CCDData(flatAvg, unit='adu')
	calibratedImages = []
	for file in files:
		filedata = fits.getdata(file)
		filedataCCD = CCDData(filedata, unit='adu')
		subtractedBias = CCDData.subtract(filedataCCD, biasAvgCCD)
		calibratedImg = CCDData.divide(subtractedBias, flatAvgCCD)
		calibratedImages.append(calibratedImg)
	return calibratedImages

#---------------------------------------
#Calibrate images if user wants to	
#---------------------------------------
calib = str(input('Would you like to calibrate your images? If so, you will need biases and flats. (y/n) '))
if calib == 'y' or calib == 'yes' or calib == 'Y' or calib == 'Yes':
	biasesBool, biasAvg, biasPath = getCalibAvg("biases")
	flatsBool, flatAvg, flatPath = getCalibAvg("flats")
	if not(biasesBool):
		print("Error: no biases found in " + biasPath + ".")
	if not(flatsBool):
		print("Error: no flats found in " + flatPath + ".")
	if biasesBool and flatsBool: 
		sortedData = calibrate(sortedfiles, biasAvg, flatAvg)
	else:
		print("Proceeding WITHOUT calibration.")
else:
	sortedData = []
	for file in sortedfiles:
		data = fits.getdata(file)
		sortedData.append(data)
		
#-----------------------------------------------
#GET REFERENCE IMAGE AND TARGET STAR COORDINATES	
#-----------------------------------------------
ref_image_name = str(input("Enter the name of a reference image: "))
data_ref = fits.getdata(ref_image_name)
r_aper = float(input("Enter the aperture radius (in pixel width): "))
r_inn = float(input("Enter the inner annulus radius (in pixel width): "))
r_out = float(input("Enter the outer annulus radius (in pixel width): "))
x_targ_ref = float(input("Enter the x coordinate for the target star in the reference image: "))
y_targ_ref = float(input("Enter the y coordinate for the target star in the reference image: "))

#-----------------------------------
#GET DATA FOR TARGET STAR
#-----------------------------------
julian_dates = []
brightnessMeasurements = []
numfiles = len(sortedfiles)

targetPixels = []
for i in range(numfiles):
	file = sortedfiles[i]
	currentFileData = sortedData[i]
	targetPixel = getPixelValue(x_targ_ref, y_targ_ref, currentFileData, data_ref, r_aper, r_inn, r_out)
	targetPixels.append(targetPixel)
	#Get julian dates
	hdul = fits.open(file)
	hdr = hdul[0].header
	MJDpresent = 'MJD-OBS' in hdr 
	if MJDpresent: 
		mjd_date = hdr['MJD-OBS']
	else:
		date = hdr['DATE-OBS']
		t = Time(date)
		mjd_date = t.mjd
	julian_dates.append(mjd_date)

#-----------------------------------
#GET DATA FOR COMPARISON STARS
#-----------------------------------
comparisonPixels = []
num_of_comparisons = int(input("How many comparison stars would you like to use? "))
for num in range(1, num_of_comparisons+1):
	x_comp_ref = float(input("Enter the x coordinate for comparison star " + str(num) + " in the reference image: "))
	y_comp_ref = float(input("Enter the y coordinate for comparison star " + str(num) + " in the reference image: "))

	comparisonStarPixels = []
	for i in range(numfiles):
		currentFileData = sortedData[i]		
		compPixel = getPixelValue(x_comp_ref, y_comp_ref, currentFileData, data_ref, r_aper, r_inn, r_out)
		comparisonStarPixels.append(compPixel)
	comparisonPixels.append(comparisonStarPixels)	
	
comparisonPixelsArray = np.array(comparisonPixels)

#-----------------------------------
#BRIGHTNESS MEASURMENTS
#-----------------------------------	
cols = len(sortedfiles)
for n in range(cols):
	comparisons_in_image = comparisonPixelsArray[:,n]
	average = np.average(comparisons_in_image)
	relativeBrightness = targetPixels[n]/average
	brightnessMeasurements.append(relativeBrightness)


#-----------------------------------
#CHECK FOR NEGATIVE MEASUREMENTS
#-----------------------------------
brightnessMeasurementsFixed = []
for relBright in brightnessMeasurements:
	brightnessMeasurementsFixed.append(relBright)

negBrightness = []
negFilename = []

for relBright in brightnessMeasurementsFixed:
	if relBright < 0:
		index = brightnessMeasurements.index(relBright)
		negBrightness.append(relBright)
		negFilename.append(sortedfiles[index])
		sortedfiles.pop(index)
		brightnessMeasurements.pop(index)
		julian_dates.pop(index)
		sortedData.pop(index)
	
numOfNeg = len(negBrightness)
if numOfNeg > 0:
	with open("NegativeMeasurements.csv", 'w', newline='') as file:
		writer = csv.writer(file)
		writer.writerow(["File name", "Relative brightness"])
		for n in range(numOfNeg):
			writer.writerow([negFilename[n], negBrightness[n]])
	print("\n" + "WARNING:")
	print("At least one of the relative brightness measurements is negative.")
	print("Look for the file 'NegativeMeasurements.csv' in your current directory to see which images have negative measurements.")
	cols = len(sortedfiles)


#-----------------------------------
#OUTPUT GRAPH AND CSV FILE
#-----------------------------------
fileName = str(input("Enter the name of the csv file you would like to save your data to: "))
		
with open(fileName, 'w', newline='') as file:
    writer = csv.writer(file)
    writer.writerow(["File name", "Julian date", "Relative brightness"])
    for n in range(cols):
    	writer.writerow([sortedfiles[n], julian_dates[n], brightnessMeasurements[n]])

titleAsk = str(input("Would you like to add a title to your graph? (y/n) "))
if titleAsk == 'y' or titleAsk == 'yes' or titleAsk == 'Y' or titleAsk == 'Yes':
	title = str(input("Enter the title of your graph: "))
	titleBool = True
else:
	titleBool = False

plt.plot(julian_dates, brightnessMeasurements,"bo")
if titleBool:
	plt.title(title)
plt.xlabel('julian date (MJD)')
plt.ylabel('relative brightness')
plt.show()





	










